@@ -1,5 +1,7 @@
+# sub-packages
+
+# libraries
 from needle_shape_sensing import (
-    tensorflow,
     cost_functions,
     fbg_signal_processing,
     geometry,
@@ -7,4 +9,5 @@ from needle_shape_sensing import (
     numerical,
     sensorized_needles,
     shape_sensing,
-)
\ No newline at end of file
+)
+
